require 'rspec'
require 'ruby_man/aabb'

# Common RubyMan namespace
module RubyMan
  describe 'AABB' do
    context 'intersection' do
      subject(:box) { AABB.new(2, 2, 8, 8) }
      it 'intersects basic cases' do
        expect(box.intersects?(AABB.new(0, 0, 4, 4))).to be true
        expect(box.intersects?(AABB.new(4, 4, 10, 10))).to be true
        expect(box.intersects?(AABB.new(0, 4, 5, 10))).to be true
        expect(box.intersects?(AABB.new(4, 0, 10, 5))).to be true
      end

      it 'intersects edges' do
        expect(box.intersects?(AABB.new(3, 0, 4, 10))).to be true
        expect(box.intersects?(AABB.new(0, 3, 10, 4))).to be true
      end

      it 'accepts floats' do
        expect(box.intersects?(AABB.new(2.1, 0, 7.9, 10))).to be true
      end

      it 'does not intersect' do
        expect(box.intersects?(AABB.new(-1000, -1000, 1, 1))).to be false
        expect(box.intersects?(AABB.new(9, 9, 1000, 1000))).to be false
        expect(box.intersects?(AABB.new(-1000, -1000, 1000, 1))).to be false
        expect(box.intersects?(AABB.new(-1000, 9, 1000, 1000))).to be false
      end

      it 'intersects when contained inside other' do
        expect(box.intersects?(AABB.new(0, 0, 10, 10))).to be true
      end

      it 'intersects when containing other' do
        expect(box.intersects?(AABB.new(4, 4, 5, 5))).to be true
      end
    end
  end
end
