require 'rspec'
require 'ruby_man/direction'

# Common RubyMan namespace
module RubyMan
  describe 'Direction' do
    context 'should work with Array booleans' do
      subject(:arr) { [Direction.left, Direction.right, Direction.up] }

      it 'works with intersection' do
        expect(arr & [Direction.right,
                      Direction.down,
                      Direction.up]).to eq [Direction.right, Direction.up]
      end

      it 'works with subtraction' do
        expect(arr - [Direction.right,
                      Direction.down,
                      Direction.up]).to eq [Direction.left]
      end

      it 'works with equal' do
        expect(arr).to eq [Direction.left, Direction.right, Direction.up]
      end
    end

    context 'functionality' do
      subject(:dir) { Direction.up }
      it { expect(dir).to eq Direction.up }
      it { expect(dir.opposite).to eq Direction.down }
      it do
        expect(dir.inverted).to eq [Direction.right,
                                    Direction.down,
                                    Direction.left]
      end
      it do
        expect(Direction.each.to_a).to eq [Direction.right,
                                           Direction.down,
                                           Direction.left,
                                           Direction.up]
      end
      it { expect(Direction.each.to_a.map(&:to_i)).to eq [0, 1, 2, 3] }
      it { expect(Direction.each.to_a.map(&:to_angle)).to eq [0, 90, 180, 270] }
      it { expect(Direction.each.to_a.map(&:rel_x)).to eq [1, 0, -1, 0] }
      it { expect(Direction.each.to_a.map(&:rel_y)).to eq [0, 1, 0, -1] }
    end
  end
end
